package main

import "fmt"
import "time"

func benchmarkMapLookup(m map[string]int, keys []string) time.Duration {
	var key string
	var start time.Time

	start = time.Now()
	for _, key = range keys {
		_ = m[key]
	}

	return time.Since(start)
}

func benchmarkSliceLookup(s []*item, keys []string) time.Duration {
	var key string
	var it *item
	var start time.Time

	start = time.Now()
	for _, key = range keys {
		for _, it = range s {
			if it.key == key {
				_ = it.value
				break
			}
		}
	}

	return time.Since(start)
}

func main() {
	var keys []string
	var m map[string]int
	var s []*item
	var numLookup = 10000000
	var tmap float64
	var tslice float64

	fmt.Println("# nb\tmap_time\tlist_time\tgain")

	for nb = 1; nb <= 40; nb ++ {
		keys = generateRandomKeys(numLookup)
		m, s = generateItems()

		mapTime := benchmarkMapLookup(m, keys)
		sliceTime := benchmarkSliceLookup(s, keys)

		tmap = float64(mapTime.Nanoseconds())/float64(numLookup)
		tslice = float64(sliceTime.Nanoseconds())/float64(numLookup)

		fmt.Printf("%d\t%.2f\t%.2f\t%.2f\n",
			nb,
			tmap,
			tslice,
			(tmap - tslice) / tmap * 100.0)
	}
}
